@@ -356,17 +356,22 @@ void PersonsModel::updateContact(const QUrl &uri)
 
     kDebug() << "Updating contact" << uri;
 
-    if (!d->contacts.contains(uri)) {
+    ContactItem *contact = d->contacts[uri];
+
+    if (!contact) {
         kWarning() << "Contact not found! Uri is" << uri;
         return;
     }
 
+    contact->clear();
+
     QString queryString = d->prepareQuery(uri);
 
     Soprano::Model *m = Nepomuk2::ResourceManager::instance()->mainModel();
     Soprano::Util::AsyncQuery *query = Soprano::Util::AsyncQuery::executeQuery(m, queryString, Soprano::Query::QueryLanguageSparql);
     query->setProperty("contactUri", QVariant(uri));
 
+
     connect(query, SIGNAL(nextReady(Soprano::Util::AsyncQuery*)),
             this, SLOT(nextReady(Soprano::Util::AsyncQuery*)));
 
@@ -634,6 +639,19 @@ KJob* PersonsModel::createPersonFromUris(const QList<QUrl> &uris)
         kWarning() << "not implemented yet";
     }
 
+    //TODO
+    //This makes the job synchronous, which is bad.. but there's already a synchronous call in this method anyway (line 591).
+    //If Soprano is blocked we already block our UI so this has little real impact
+
+    //we need some changes to make sure that if this method is called twice in quick succession for the same contacts
+    //otherwise we can end up making attaching a single contact to two different PIMO:Person's as we check if the contact is owned by a person whilst a merge
+    //job is happening asyncronously. This effectively breaks the database
+    //This cannot be fixed without a _signifcant_ refactor.
+
+    if (job) {
+        job->exec();
+    }
+
     return job;
 }
 
